@@ -410,11 +410,19 @@ Ztring MediaInfo_Internal::Inform()
 
     #if defined(MEDIAINFO_XML_YES)
         if (XML || XML_0_7_78_MA || XML_0_7_78_MI)
+        {
             Retour=Ztring().From_UTF8(To_XML(*Node_Main, 0, false, false));
+            delete Node_Main;
+            delete Node_MI;
+        }
     #endif //MEDIAINFO_XML_YES
     #if defined(MEDIAINFO_JSON_YES)
         if (JSON)
+        {
             Retour=__T("{\n")+Ztring().From_UTF8(To_JSON(*Node_Main, 0, false, false))+__T("\n}");
+            delete Node_Main;
+            delete Node_MI;
+        }
     #endif //MEDIAINFO_JSON_YES
 
     Retour.FindAndReplace(__T("\\"), __T("|SC1|"), 0, Ztring_Recursive);
@@ -458,6 +466,16 @@ Ztring MediaInfo_Internal::Inform()
 
 //---------------------------------------------------------------------------
 #if defined(MEDIAINFO_TEXT_YES) || defined(MEDIAINFO_HTML_YES) || defined(MEDIAINFO_XML_YES) || defined(MEDIAINFO_CSV_YES) || defined(MEDIAINFO_CUSTOM_YES)
+#if defined(MEDIAINFO_XML_YES) || defined(MEDIAINFO_JSON_YES)
+namespace
+{
+struct nested
+{
+    std::vector<Node*>* Target;
+    Ztring Name;
+};
+}
+#endif //MEDIAINFO_XML_YES || MEDIAINFO_JSON_YES
 Ztring MediaInfo_Internal::Inform (stream_t StreamKind, size_t StreamPos, bool IsDirect)
 {
     //Integrity
@@ -477,7 +495,6 @@ Ztring MediaInfo_Internal::Inform (stream_t StreamKind, size_t StreamPos, bool I
         bool XML=false;
         bool XML_0_7_78=false;
         bool JSON=false;
-        std::vector<Node*> Nodes;
         #endif //MEDIAINFO_XML_YES || MEDIAINFO_JSON_YES
 
         #if defined(MEDIAINFO_HTML_YES)
@@ -514,13 +531,18 @@ Ztring MediaInfo_Internal::Inform (stream_t StreamKind, size_t StreamPos, bool I
              Text=false;
         #endif //defined(MEDIAINFO_CSV_YES)
         #endif //defined(MEDIAINFO_TEXT_YES) && (defined(MEDIAINFO_HTML_YES) || defined(MEDIAINFO_XML_YES) || defined(MEDIAINFO_CSV_YES))
-        size_t Size=Count_Get(StreamKind, StreamPos);
-        bool IsExtra=false;
 
         #if defined(MEDIAINFO_XML_YES) || defined(MEDIAINFO_JSON_YES)
         std::vector<Node*> Fields;
+        std::vector<Node*>* Fields_Current=&Fields;
+        std::vector<Node*>* Extra=NULL;
+        std::vector<nested> Nested;
+        bool ExpandSub=Config.File_ExpandSubs_Get();
+        if ((XML_0_7_78 || JSON) && ExpandSub)
+            Config.File_ExpandSubs_Set(false);
         #endif //MEDIAINFO_XML_YES || MEDIAINFO_JSON_YES
 
+        size_t Size = Count_Get(StreamKind, StreamPos);
         for (size_t Champ_Pos=0; Champ_Pos<Size; Champ_Pos++)
         {
             //Pour chaque champ
@@ -550,11 +572,11 @@ Ztring MediaInfo_Internal::Inform (stream_t StreamKind, size_t StreamPos, bool I
             {
                 #if defined(MEDIAINFO_XML_YES) || defined(MEDIAINFO_JSON_YES)
                 //Extra
-                if ((XML_0_7_78 || JSON) && !IsExtra && Champ_Pos>=Stream[StreamKind][StreamPos].size())
+                if ((XML_0_7_78 || JSON) && !Extra && Champ_Pos>=Stream[StreamKind][StreamPos].size())
                 {
-                     IsExtra=true;
-                     Node* Node_Extra=new Node("extra");
-                     Fields.push_back(Node_Extra);
+                    Node* Node_Extra=new Node("extra");
+                    Fields.push_back(Node_Extra);
+                    Fields_Current=Extra=&Node_Extra->Childs;
                 }
                 #endif //defined(MEDIAINFO_XML_YES || MEDIAINFO_JSON_YES
 
@@ -566,6 +588,60 @@ Ztring MediaInfo_Internal::Inform (stream_t StreamKind, size_t StreamPos, bool I
                 #endif //MEDIAINFO_XML_YES || MEDIAINFO_JSON_YES
 
                     Nom=Get((stream_t)StreamKind, StreamPos, Champ_Pos, Info_Name); //Texte n'existe pas
+
+                //Subs
+                #if defined(MEDIAINFO_XML_YES) || defined(MEDIAINFO_JSON_YES)
+                if (XML || JSON)
+                {
+                    const Ztring& NextName=Get((stream_t)StreamKind, StreamPos, Champ_Pos+1, Info_Name);
+                    size_t Nom_ToErase=0;
+
+                    if (!Nested.empty())
+                    {
+                        nested& LastNested=Nested[Nested.size()-1];
+                        while(!Nested.empty()
+                         && !(Nom.size()>LastNested.Name.size() && !Nom.rfind(LastNested.Name, LastNested.Name.size()) && Nom[LastNested.Name.size()]==__T(' ')))
+                        {
+                            Fields_Current=LastNested.Target;
+                            Nested.pop_back();
+                            if (Nested.empty())
+                                break;
+                            LastNested=Nested[Nested.size()-1];
+                        }
+                        if (Nested.empty())
+                        {
+                            if (Extra)
+                                Fields_Current=Extra;
+                            else
+                                Fields_Current=&Fields;
+                        }
+                        else
+                            Nom_ToErase=LastNested.Name.size()+1;
+                    }
+
+                    if (NextName.size()>Nom.size() && !NextName.rfind(Nom, Nom.size()) && NextName[Nom.size()]==__T(' '))
+                    {
+                        string SubName=Nom.To_UTF8();
+                        size_t Space=SubName.rfind(' ');
+                        if (Space!=(size_t)-1)
+                            SubName.erase(0, Space+1);
+                        size_t NumbersPos=SubName.find_last_not_of("0123456789");
+                        if (NumbersPos!=(size_t)-1)
+                            SubName.resize(NumbersPos+1);
+                        bool IsArray=(NextName.size()==Nom.size()+4 && NextName.find(__T(" Pos"), Nom.size())==Nom.size());
+                        Node* Node_Sub=new Node(SubName.c_str(), IsArray);
+                        Fields_Current->push_back(Node_Sub);
+                        Nested.resize(Nested.size()+1);
+                        nested& LastNested=Nested[Nested.size()-1];
+                        LastNested.Name=Nom;
+                        LastNested.Target=Fields_Current;
+                        Fields_Current=&Node_Sub->Childs;
+                        continue;
+                    }
+                    Nom.erase(0, Nom_ToErase);
+                }
+                #endif // defined(MEDIAINFO_XML_YES) || defined(MEDIAINFO_JSON_YES)
+
                 #if defined(MEDIAINFO_TEXT_YES) && (defined(MEDIAINFO_HTML_YES) || defined(MEDIAINFO_XML_YES) || defined(MEDIAINFO_JSON_YES) || defined(MEDIAINFO_CSV_YES))
                 if (Text)
                 #endif //defined(MEDIAINFO_TEXT_YES) && (defined(MEDIAINFO_HTML_YES) || defined(MEDIAINFO_XML_YES) || defined(MEDIAINFO_JSON_YES) || defined(MEDIAINFO_CSV_YES))
@@ -607,8 +683,13 @@ Ztring MediaInfo_Internal::Inform (stream_t StreamKind, size_t StreamPos, bool I
                 #if defined(MEDIAINFO_HTML_YES)
                 if (HTML)
                 {
+                    size_t Level=Nom.find_first_not_of(__T(" "));
+                    Ztring Prefix;
+                    for (size_t Pos=0; Pos<Level; Pos++)
+                        Prefix+=__T("&nbsp;");
+
                     Retour+=__T("  <tr>\n    <td><i>");
-                    Retour+=Nom;
+                    Retour+=Prefix+Nom.TrimLeft();
                     Retour+=__T(" :</i></td>\n    <td colspan=\"3\">");
                     Retour+=Valeur;
                     Retour+=__T("</td>\n  </tr>");
@@ -656,7 +737,7 @@ Ztring MediaInfo_Internal::Inform (stream_t StreamKind, size_t StreamPos, bool I
                         Node_Current->Value="(Binary data)";
                     else
                         Node_Current->Value=Valeur.To_UTF8();
-                    IsExtra?Fields.back()->Childs.push_back(Node_Current):Fields.push_back(Node_Current);
+                    Fields_Current->push_back(Node_Current);
 
                     if (!Format_Profile_More.empty())
                     {
@@ -666,14 +747,14 @@ Ztring MediaInfo_Internal::Inform (stream_t StreamKind, size_t StreamPos, bool I
                         if (SeparatorPos!=string::npos)
                         {
                             Node_Current=new Node("Format_Level", Ztring(Format_Profile_More.substr(0, SeparatorPos)).To_UTF8());
-                            IsExtra?Fields.back()->Childs.push_back(Node_Current):Fields.push_back(Node_Current);
+                            Fields_Current->push_back(Node_Current);
                             Node_Current=new Node("Format_Tier", Ztring(Format_Profile_More.substr(SeparatorPos+1)).To_UTF8());
-                            IsExtra?Fields.back()->Childs.push_back(Node_Current):Fields.push_back(Node_Current);
+                            Fields_Current->push_back(Node_Current);
                         }
                         else
                         {
                             Node_Current=new Node("Format_Level", Format_Profile_More.To_UTF8());
-                            IsExtra?Fields.back()->Childs.push_back(Node_Current):Fields.push_back(Node_Current);
+                            Fields_Current->push_back(Node_Current);
                         }
                     }
                 }
@@ -698,39 +779,32 @@ Ztring MediaInfo_Internal::Inform (stream_t StreamKind, size_t StreamPos, bool I
             }
         }
         #if defined(MEDIAINFO_XML_YES) || defined(MEDIAINFO_JSON_YES)
+        if (ExpandSub)
+            Config.File_ExpandSubs_Set(true);
         for (size_t Field=0; Field < Fields.size(); Field++)
         {
-            if (Fields[Field]->Name == "extra")
-            {   std::string TmpRetour;
-                for (size_t Field2=0; Field2 < Fields[Field]->Childs.size(); Field2++)
-                    if (XML)
-                        TmpRetour+=To_XML(*(Fields[Field]->Childs[Field2]), 1, false, false);
-                    else if (JSON)
-                        TmpRetour+=To_JSON(*(Fields[Field]->Childs[Field2]), 1, false, false)+(Field2<Fields[Field]->Childs.size()-1?",\n":"");
-
-                Fields[Field]->Childs.clear();
-                Node* Node_Raw=new Node();
-                Node_Raw->RawContent=TmpRetour;
-                Fields[Field]->Childs.push_back(Node_Raw);
-            }
-
             if (XML)
                 Retour+=Ztring().From_UTF8(To_XML(*(Fields[Field]), 1, false, false));
             else if (JSON)
                 Retour+=Ztring().From_UTF8(To_JSON(*(Fields[Field]), 1, false, false)+(Field<Fields.size()-1?",\n":""));
+            delete Fields[Field];
         }
-
         #endif //defined(MEDIAINFO_XML_YES) || defined(MEDIAINFO_JSON_YES)
         ConvertRetour(Retour);
 
-        Retour.FindAndReplace(__T("|SC1|"), __T("\\"), 0, Ztring_Recursive);
+        #if defined(MEDIAINFO_JSON_YES)
+            if (JSON)
+                Retour.FindAndReplace(__T("|SC1|"), __T("\\\\"), 0, Ztring_Recursive);
+            else
+                Retour.FindAndReplace(__T("|SC1|"), __T("\\"), 0, Ztring_Recursive);
+        #else
+            Retour.FindAndReplace(__T("|SC1|"), __T("\\"), 0, Ztring_Recursive);
+        #endif
+
         return Retour;
     }
 
     Ztring Retour=MediaInfoLib::Config.Inform_Get(Get(StreamKind, 0, __T("StreamKind"), Info_Text));
-    ZtringList Info;
-
-    Info=Stream[StreamKind][StreamPos];
 
     //Special characters
     Retour.FindAndReplace(__T("\\\\"), __T("|SC1|"), 0, Ztring_Recursive);
@@ -772,7 +846,7 @@ Ztring MediaInfo_Internal::Inform (stream_t StreamKind, size_t StreamPos, bool I
         size_t Pos=MediaInfoLib::Config.Info_Get(StreamKind).Find(Elements(0));
         if (Pos!=std::string::npos)
         {
-            if (Info(Pos).size()>0)
+            if (Get(StreamKind, StreamPos, Pos).size()>0)
                 Elements_Index=1;
             else
                 Elements_Index=2;
@@ -784,7 +858,14 @@ Ztring MediaInfo_Internal::Inform (stream_t StreamKind, size_t StreamPos, bool I
         while (Elements(Elements_Index).SubString(__T("%"), __T("%")).size()>0)
         {
             Ztring ToReplace=Elements(Elements_Index).SubString(__T("%"), __T("%"));
-            Ztring ReplacedBy=Info(MediaInfoLib::Config.Info_Get(StreamKind).Find(ToReplace));
+            Ztring ReplacedBy=Get(StreamKind, StreamPos, ToReplace);
+            ReplacedBy.FindAndReplace(__T("\\"), __T("|SC1|"), 0, Ztring_Recursive);
+            ReplacedBy.FindAndReplace(__T("["), __T("|SC2|"), 0, Ztring_Recursive);
+            ReplacedBy.FindAndReplace(__T("]"), __T("|SC3|"), 0, Ztring_Recursive);
+            ReplacedBy.FindAndReplace(__T(","), __T("|SC4|"), 0, Ztring_Recursive);
+            ReplacedBy.FindAndReplace(__T(";"), __T("|SC5|"), 0, Ztring_Recursive);
+            ReplacedBy.FindAndReplace(__T("("), __T("|SC6|"), 0, Ztring_Recursive);
+            ReplacedBy.FindAndReplace(__T(")"), __T("|SC7|"), 0, Ztring_Recursive);
             ToReplace=Ztring(__T("%"))+ToReplace+Ztring(__T("%"));
             Elements(Elements_Index).FindAndReplace(ToReplace, ReplacedBy);
         }
@@ -799,25 +880,20 @@ Ztring MediaInfo_Internal::Inform (stream_t StreamKind, size_t StreamPos, bool I
     {
         Ztring Crochets=Retour.SubString(__T("["), __T("]"));
         Ztring ValueToFind=Crochets.SubString(__T("%"), __T("%"));
-        size_t ValueToFind_Pos=MediaInfoLib::Config.Info_Get(StreamKind).Find(ValueToFind);
         Ztring ARemplacer=Ztring(__T("[")+Crochets+__T("]"));
-        if (ValueToFind_Pos!=Error)
         {
-            Ztring RemplacerPar=Info(ValueToFind_Pos);
+            Ztring RemplacerPar= Get(StreamKind, StreamPos, ValueToFind);
             if (RemplacerPar.empty())
                 Retour.FindAndReplace(ARemplacer, Ztring());
             else
             {
                 //Formate l'interieur
                 Ztring ATraiter=Crochets;
-                Ztring Crochets_ARemplacer=Ztring(__T("%")+ATraiter.SubString(__T("%"), __T("%")))+__T("%");
-                Ztring Crochets_RemplacerPar=Info(MediaInfoLib::Config.Info_Get(StreamKind).Find(ATraiter.SubString(__T("%"), __T("%"))));
-                ATraiter.FindAndReplace(Crochets_ARemplacer, Crochets_RemplacerPar);
+                Ztring Crochets_ARemplacer=__T("%")+ValueToFind+__T("%");
+                ATraiter.FindAndReplace(Crochets_ARemplacer, RemplacerPar);
                 Retour.FindAndReplace(ARemplacer, ATraiter);
             }
         }
-        else
-            Retour.FindAndReplace(ARemplacer, Ztring());
     }
 
     //Gestion %xxx%
